@ECHO OFF



REM

REM        Build all of the browser implementation

REM

REM               (c) 1998 ADI Limited

REM Written by Justin Couch couch@ccis.adisys.com.au

REM

REM There is no restrictions on this. Use it however you feel!

REM

REM The script allows you to build all of the application in a single hit

REM or only parts of it. This is by no means a standard Makefile, it basically

REM only compiles *.java in the required directory

REM

REM The usage of this file is

REM

REM c:\src_dir>build [options] [build type]

REM    Options are:

REM      -help     Print out the usage message

REM      -docdir   Directory to place javadoc docs

REM      -javadoc  Create the Javadoc rather than compile the classes

REM

REM    build types are

REM      <default> Build everything except ISO EAI classes

REM      ALL       All classes, including ISO EAI classes

REM      EAI       The ISO eai classes

REM      IMPL      The Java3D loader implementation classes

REM      JSAI_IMPL The Script binding classes to the J3D browser

REM      EAI_IMPL  The EAI implementation for the J3D browser

REM      CONF      The EAI conformance testing code

REM      TESTS     The EAI conformance test plugins

REM

REM

REM When executing a javadoc statement, if no docdir is specified, a

REM directory called doc is created under the CWD and everything placed

REM in that as the TLD.

REM



SETLOCAL



set DOCUMENT_DIR=docs



REM

REM The set of packages that we need to compile

REM

REM Classes and packages for the eai ISO class definitions

REM



set EAI_C=src\vrml\eai\*.java

set EAI_C=%EAI_C% src\vrml\eai\field\*.java

set EAI_C=%EAI_C% src\vrml\eai\event\*.java



set EAI_P=vrml.eai

set EAI_P=%EAI_P% vrml.eai.field

set EAI_P=%EAI_P% vrml.eai.event



REM

REM Classes and packages for the J3D loader and core implementation

REM



set J3D_IMPL_C=src\com\sun\j3d\loaders\vrml97\*.java

set J3D_IMPL_C=%J3D_IMPL_C% src\com\sun\j3d\loaders\vrml97\impl\*.java



set J3D_IMPL_P=src\com.sun.j3d.loaders.vrml97

set J3D_IMPL_P=%J3D_IMPL_P% src\com.sun.j3d.loaders.vrml97.impl



REM

REM Classes and packages for the J3D JSAI implementation

REM



set JSAI_IMPL_C=src\vrml\*.java

set JSAI_IMPL_C=%JSAI_IMPL_C% src\vrml\field\*.java

set JSAI_IMPL_C=%JSAI_IMPL_C% src\vrml\node\*.java



set JSAI_IMPL_P=src\vrml

set JSAI_IMPL_P=%JSAI_IMPL_P% src\vrml.field

set JSAI_IMPL_P=%JSAI_IMPL_P% src\vrml.node



REM

REM Classes and packages for the EAI implementation for the core browser

REM



set EAI_IMPL_C=org\vrml\java3d\eai\*.java

set EAI_IMPL_P=org.vrml.java3d.eai



REM

REM Classes and packages for the EAI conformance testing application

REM



set EAI_CONF_APP_C=org\vrml\eai\conftest\*.java

set EAI_CONF_APP_C=%EAI_CONF_APP_C% org\vrml\eai\gui\text\*.java

set EAI_CONF_APP_C=%EAI_CONF_APP_C% BrowserTest.java



set EAI_CONF_APP_P=org.vrml.eai.conftest

set EAI_CONF_APP_P=%EAI_CONF_APP_P% org.vrml.eai.gui.text



REM

REM Classes and packages for the EAI testing plugins in the conformance app

REM



set EAI_TESTS_C=org\vrml\eai\conftest\tests\*.java



set EAI_TESTS_P=org.vrml.eai.conftest.tests



REM

REM Set the compile type as a variable. If there is no arguments then the

REM batch file treats this as not being defined. If it is not defined then

REM it defaults to running all of the servers. The reason for going about

REM things this way is because the IF DEFINED does not seem to be able to

REM handle command line args directly. :(

REM



REM Changed by Erik Pagel , erik.pagel@olympus-software.com , 25.02.1999

REM add a new Parameter for javac. For static uncomed the secound line

SET COMPILE_OPTIONS = %3

REM SET COMPILE_OPTIONS=-deprecation





SET FIRST_ARG=%1



IF DEFINED FIRST_ARG GOTO HAS_ARG_1



SET FIRST_ARG=default

SET ACTION=compile

GOTO VALID_ARG



:PRINT_HEADER

  cls



  echo *******************************************************************************

  echo *                                                                             *

  echo *                   Java3D VRML Browser build script                          *

  echo *                                                                             *

  echo *                         (c) 1998 ADI Ltd                                    *

  echo *                                                                             *

  echo *******************************************************************************

  echo.

  echo.



REM Changed by Erik Pagel , erik.pagel@olympus-software.com , 25.02.1999

REM First we have to change the Name of MovieTexture.java because it's not implemented now !

echo Changing name of MovieTexture.java to MovieTexture.jav_ because it's not 

echo implemented jet and it confuse javac

IF EXIST src\com\sun\j3d\loaders\vrml97\impl\MovieTexture.java ren src\com\sun\j3d\loaders\vrml97\impl\MovieTexture.java src\com\sun\j3d\loaders\vrml97\impl\MovieTexture.jav_





  IF /I %ACTION% == compile GOTO %FIRST_ARG%

  IF /I %ACTION% == docs GOTO DOC_PROCESS



  REM should never get here!

  echo Invalid action type - exiting

  GOTO END



REM

REM Process The first argument

REM



:HAS_ARG_1

  IF /I %FIRST_ARG% == -javadoc  GOTO VALID_ARG

  IF /I %FIRST_ARG% == ALL       GOTO VALID_ARG

  IF /I %FIRST_ARG% == EAI       GOTO VALID_ARG

  IF /I %FIRST_ARG% == IMPL      GOTO VALID_ARG

  IF /I %FIRST_ARG% == JSAI_IMPL GOTO VALID_ARG

  IF /I %FIRST_ARG% == EAI_IMPL  GOTO VALID_ARG

  IF /I %FIRST_ARG% == CONF      GOTO VALID_ARG

  IF /I %FIRST_ARG% == TESTS     GOTO VALID_ARG

  IF %FIRST_ARG% == ? GOTO HELP

  IF /I %FIRST_ARG% == -help GOTO HELP



  GOTO HELP



:VALID_ARG

  SET ACTION=compile

  GOTO PRINT_HEADER



REM

REM The default compile option. This compiles everything except ISO classes

REM



:DEFAULT

  echo Default compile called

  set COMPILE_CLASSES= %J3D_IMPL_C% %JSAI_IMPL_C%

  set COMPILE_CLASSES=%COMPILE_CLASSES% %EAI_IMPL_C% %EAI_CONF_APP_C%

  set COMPILE_CLASSES=%COMPILE_CLASSES% %EAI_TESTS_C%

  GOTO COMPILE



:ALL

  echo Compiling all

  set COMPILE_CLASSES= %EAI_C% %J3D_IMPL_C% %JSAI_IMPL_C%

  set COMPILE_CLASSES=%COMPILE_CLASSES% %EAI_IMPL_C% %EAI_CONF_APP_C%

  set COMPILE_CLASSES=%COMPILE_CLASSES% %EAI_TESTS_C%

  GOTO COMPILE



:EAI

  echo Compiling EAI ISO classes

  set COMPILE_CLASSES= %EAI_C%

  GOTO COMPILE



:IMPL

  echo Compiling J3D implementation

  set COMPILE_CLASSES= %J3D_IMPL_C%

  GOTO COMPILE



:JSAI_IMPL

  echo Compiling JSAI implementation

  set COMPILE_CLASSES= %JSAI_IMPL_C%

  GOTO COMPILE



:EAI_IMPL

  echo Compiling EAI implementation

  set COMPILE_CLASSES= %EAI_IMPL_C%

  GOTO COMPILE



:CONF

  echo Compiling conformance testing app

  set COMPILE_CLASSES= %EAI_CONF_APP_C%

  GOTO COMPILE



:TESTS

  echo Compiling conformance tests

  set COMPILE_CLASSES= %EAI_TESTS_C%

  GOTO COMPILE



REM

REM Compile the classes using javac. Requires the use of the COMPILE_CLASSES

REM variable to work

REM



:COMPILE

  javac %COMPILE_OPTIONS%  %COMPILE_CLASSES%

GOTO END





REM

REM Documentation generation processing

REM



REM

REM Javadoc generation option. The same arguments build types are available

REM for javadoc as the standard compile.

REM

:-javadoc

  echo Creating javadoc!



  set SECOND_ARG=%2

  IF DEFINED SECOND_ARG GOTO HAS_ARG_2



  set SECOND_ARG=default

  GOTO DOC_VALID_ARG



:HAS_ARG_2

  IF /I %SECOND_ARG% == -docdir   GOTO SET_DOC_DIR

  IF /I %SECOND_ARG% == ALL       GOTO DOC_VALID_ARG

  IF /I %SECOND_ARG% == EAI       GOTO DOC_VALID_ARG

  IF /I %SECOND_ARG% == IMPL      GOTO DOC_VALID_ARG

  IF /I %SECOND_ARG% == JSAI_IMPL GOTO DOC_VALID_ARG

  IF /I %SECOND_ARG% == EAI_IMPL  GOTO DOC_VALID_ARG

  IF /I %SECOND_ARG% == CONF      GOTO DOC_VALID_ARG

  IF /I %SECOND_ARG% == TESTS     GOTO DOC_VALID_ARG



  GOTO HELP



REM

REM Set the document directory. This should be the 3rd argument from the

REM commandline (it must be space separated from the -docdir arg). Then

REM we want to recurse back to the actual javadoc command. We attempt to

REM read in the next argument (#4). If it is not set, then it assumes that

REM we want the default javadoc type as well. We reset SECOND_ARG to the

REM newest argument and recall HAS_ARG_2 to continue the processing. Since

REM SECOND_ARG is now no longer set to -docdir then we won't end up with

REM loops unless some dickhead puts two on the command line....

REM



:SET_DOC_DIR

  SET DOCUMENT_DIR=%3

  SET SECOND_ARG=%4



  IF DEFINED SECOND_ARG GOTO HAS_ARG_2



  SET SECOND_ARG=default

  GOTO DOC_VALID_ARG



:DOC_VALID_ARG

  SET ACTION=docs

  GOTO PRINT_HEADER



:DOC_PROCESS

  SET SECOND_ARG=%SECOND_ARG%_DOC

  GOTO %SECOND_ARG%





:DEFAULT_DOC

  echo Default javadoc called

  set DOC_PKGS= %J3D_IMPL_P% %JSAI_IMPL_P%

  set DOC_PKGS=%DOC_PKGS% %EAI_IMPL_P% %EAI_CONF_APP_P%

  set DOC_PKGS=%DOC_PKGS% %EAI_TESTS_P%

  GOTO COMPILE_DOCS



:ALL_DOC

  echo Javadocing all

  set DOC_PKGS= %EAI_P% %J3D_IMPL_P% %JSAI_IMPL_P%

  set DOC_PKGS=%DOC_PKGS% %EAI_IMPL_P% %EAI_CONF_APP_P%

  set DOC_PKGS=%DOC_PKGS% %EAI_TESTS_P%

  GOTO COMPILE_DOCS



:EAI_DOC

  echo Javadocing EAI ISO classes

  set DOC_PKGS= %EAI_P%

  GOTO COMPILE_DOCS



:IMPL_DOC

  echo Javadocing J3D implementation

  set DOC_PKGS= %J3D_IMPL_P%

  GOTO COMPILE_DOCS



:JSAI_IMPL_DOC

  echo Javadocing JSAI implementation

  set DOC_PKGS= %JSAI_IMPL_P%

  GOTO COMPILE_DOCS



:EAI_IMPL_DOC

  echo Javadocing EAI implementation

  set DOC_PKGS= %EAI_IMPL_P%

  GOTO COMPILE_DOCS



:CONF_DOC

  echo Javadocing conformance testing app

  set DOC_PKGS= %EAI_CONF_APP_P%

  GOTO COMPILE_DOCS



:TESTS_DOC

  echo Javadocing conformance tests

  set DOC_PKGS= %EAI_TESTS_P%

  GOTO COMPILE_DOCS



REM

REM Finally get around to documenting everything with javadoc. It does a

REM check to see if the document directory already exists. If it doesn't

REM then it creates it beforehand, just to make sure we don't get barfs

REM from javadoc. It requires the use of DOC_PKGS to work correctly

REM



:COMPILE_DOCS

  IF NOT EXIST %DOCUMENT_DIR% MD %DOCUMENT_DIR%

  javadoc -d %DOCUMENT_DIR% %DOC_PKGS%

  GOTO END



REM

REM Echo out the help/usage message and exit

REM

REM Changed by Erik Pagel , erik.pagel@olympus-software.com , 25.02.1999

REM add help for third parameter

:HELP

  echo usage: build options build_type compiler_switch

  echo          options: -help or -javadoc -docdir

  echo          build_type all, eai, impl, jsai_impl, eai_impl, conf, tests

  echo			compiler_switch any legal javac parameter

  echo.



:END

REM Changed by Erik Pagel , erik.pagel@olympus-software.com , 25.02.1999

REM To leave the source tree unchanged, we change the name of MovieTexture back to MovieTexture.java

echo Changing name of MovieTexture.jav_ back to MovieTexture.java 

echo to leave the source unchanged

IF EXIST src\com\sun\j3d\loaders\vrml97\impl\MovieTexture.jav_ ren src\com\sun\j3d\loaders\vrml97\impl\MovieTexture.jav_ src\com\sun\j3d\loaders\vrml97\impl\MovieTexture.java



ENDLOCAL



REM End of script!

