/*
 *      @(#)VrmlTokenManager.java 1.5 98/11/05 20:35:32
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 *
 * @Author: Rick Goldberg
 * @Author: Doug Gehringer
 */


package com.sun.j3d.loaders.vrml97.impl;

import java.util.Hashtable;

/**
 * Derived from a token manager produced by JavaCC, then simplified and 
 * customized.
 * The generated code is used to parse the input stream into single character 
 * literals and the psuedo token MULTI_CHAR.
 * The MULTI_CHAR strings are further massaged into tokens by analyzing
 * the string:
 *    switch(first char) {
 *      case '"': return STRING_LITERAL;
 * 	case numeric: return NUMERIC_LITERAL;
 * 	case alphabetic:
 * 	  lookup string in hash table of static token strings or use 
 *	  lex state to return the correct dynamic token type
 * Note: VRML allows all non-numeric, non-whitespace, non-control chars to 
 * be used in ids, i.e. '%!^~"
 *
 */
public class VrmlTokenManager implements ParserConstants, TokenManager {

    public String[] jjstrLiteralImages;
    int jjmatchedPos;
    int jjmatchedKind;

    private VrmlCharStream input_stream;
    protected char curChar;
    private Hashtable tokenStringMap;
    private Loader loader;

    int[] tokenState;
    int lexMode = DEFAULT;	 

    static final int DEFAULT = 0;
    static final int USE_ID = 1;
    static final int ROUTE_ID = 2;
    static final int FIELD_ID = 3; 
    static final int FIELD_NAME = 4; 

    static final int NO_MATCH =   0x7fffffff;
    static final int MULTI_CHAR = 0x7f000000; 

    public VrmlTokenManager(VrmlCharStream stream, Loader loader) {
	this.loader = loader;
	if (VrmlCharStream.staticFlag)
	    throw new Error("ERROR: Cannot use a static CharStream class with" +
		" a non-static lexical analyzer.");
	input_stream = stream;

	// build up the literal images for the single character tokens
	jjstrLiteralImages = new String[tokenImage.length];
	jjstrLiteralImages[LBRACE] = "{";
	jjstrLiteralImages[RBRACE] = "}";
	jjstrLiteralImages[LBRACKET] = "[";
	jjstrLiteralImages[RBRACKET] = "]";

	// build a hash table for the static string tokens
	tokenStringMap = new Hashtable();
	for (int i = 0; i < tokenImage.length; i++) {
	    if (tokenImage[i].charAt(0) != '<') {
		String tokenString = 
			tokenImage[i].substring(1, tokenImage[i].length()-1);
		tokenStringMap.put(tokenString, new Integer(i));
	    }
	}

	// build a table for changing the lex state
	tokenState = new int[tokenImage.length];
	for (int i = 0; i < tokenImage.length; i++) {
	    tokenState[i] = 0;
	}
	tokenState[DEF] = USE_ID;
	tokenState[USE] = USE_ID;
	tokenState[PROTO] = USE_ID;
	tokenState[ROUTE] = ROUTE_ID;
	tokenState[TO] = ROUTE_ID;
	tokenState[IS] = FIELD_NAME;
	tokenState[SFBOOL] = FIELD_NAME;
	tokenState[SFCOLOR] = FIELD_NAME;
	tokenState[SFFLOAT] = FIELD_NAME;
	tokenState[SFINT32] = FIELD_NAME;
	tokenState[SFNODE] = FIELD_NAME;
	tokenState[SFROTATION] = FIELD_NAME;
	tokenState[SFSTRING] = FIELD_NAME;
	tokenState[SFTIME] = FIELD_NAME;
	tokenState[SFVEC2F] = FIELD_NAME;
	tokenState[SFVEC3F] = FIELD_NAME;
	tokenState[MFCOLOR] = FIELD_NAME;
	tokenState[MFFLOAT] = FIELD_NAME;
	tokenState[MFINT32] = FIELD_NAME;
	tokenState[MFNODE] = FIELD_NAME;
	tokenState[MFROTATION] = FIELD_NAME;
	tokenState[MFSTRING] = FIELD_NAME;
	tokenState[MFTIME] = FIELD_NAME;
	tokenState[MFVEC2F] = FIELD_NAME;
	tokenState[MFVEC3F] = FIELD_NAME;

    }

    public void ReInit(VrmlCharStream stream) {
	jjmatchedPos = 0;
	input_stream = stream;
    }

    private final int jjStopAtPos(int pos, int kind) {
	jjmatchedKind = kind;
	jjmatchedPos = pos;
	return pos + 1;
    }

    private final int jjMatchToken() {
	switch(curChar) {
	  case 91:
	    return jjStopAtPos(0, LBRACKET);
	  case 93:
	    return jjStopAtPos(0, RBRACKET);
	  case 123:
	    return jjStopAtPos(0, LBRACE);
	  case 125:
	    return jjStopAtPos(0, RBRACE);
	  default :
	    return jjMatchMultiCharToken(0, 0);
	}
    }

    // curChar is not a single char token or SKIP. If part of a MULTI_CHAR, 
    // continue adding chars until non-MULTI_CHAR char is found
    private final int jjMatchMultiCharToken(int startState, int curPos) {
	boolean match = false;
	for (;;) {
	    if (curChar < 64) {
		long l = 1L << curChar;
		// matches chars: '!', '%', '(', ')', '*', '+', '-', '/',
		// '.', '0-9', ':', ';', '<', '=', '>', '?' 
		if ((0xffffef2000000000L & l) != 0L)
		    match = true;
	    } else if (curChar < 128) {
		long l = 1L << (curChar & 077);
		// matches chars: '@', 'A-Z', '^', '_', ''', 'a-z', '|', '~'
		if ((0x57ffffffd7ffffffL & l) != 0L)
		    match = true;
	    }
	    if (match) {
		jjmatchedKind = MULTI_CHAR;
		jjmatchedPos = curPos;
	    }
	    ++curPos;
	    if (!match) {
		return curPos;
	    }
	    try { 
		curChar = input_stream.readChar(); 
		match = false;
	    } catch(java.io.IOException e) { 
		return curPos; 
	    }
	}
    }

    private final Token jjFillToken(String image) {
	//System.out.println("token image: \""+ image + "\"");
	Token t = Token.newToken(jjmatchedKind);
	t.kind = jjmatchedKind;
	t.image = image;
	t.beginLine = input_stream.getBeginLine();
	t.beginColumn = input_stream.getBeginColumn();
	t.endLine = input_stream.getEndLine();
	t.endColumn = input_stream.getEndColumn();
	return t;
    }

    private final Token jjFillToken() {
	String im = jjstrLiteralImages[jjmatchedKind];
	String image = (im == null) ? input_stream.GetImage() : im;
	return jjFillToken(image);
    }
     
    private final Token processGeneralString(String image) {
        switch (lexMode) {
	  case DEFAULT:
	    Object tokenObj = tokenStringMap.get(image);
	    if (tokenObj != null) {
		jjmatchedKind = ((Integer)tokenObj).intValue();
		lexMode = tokenState[jjmatchedKind];
	    } else {
		jjmatchedKind = PROTO_IDENTIFIER;
	    }
	    return jjFillToken(image);
	  case USE_ID:
	    jjmatchedKind = USE_IDENTIFIER;
	    lexMode = DEFAULT;
	    return jjFillToken(image);
	  case ROUTE_ID:
	    // this case is a bit funky.  The string now contains three tokens:
	    // ROUTE_USE_ID DOT FNAME (this is because the a MULT_CHAR pseudo
	    // token can contain a '.', which splits the string into separate
	    // tokens in this case, but is part of the token image in the 
	    // case of a floating point number (the case we want to optimize).
	    // Resolve by finding the '.' and splitting the image into
	    // three strings. Then make three tokens and link them together.
	    // Other ways to solve this: make ROUTE_ID token in the .jj file
	    // which has the '.' inside it (move the split up to the parse
	    // level), or add a lex state to the simple lexer to generate 
	    // separate MULT_CHAR tokens for the ROUTE_USE_ID and FNAME
	    int dotIndex = image.indexOf('.');
	    if (dotIndex == -1) {
		// hopefully this is not because there is a space before the
		// dot.  If so (and this is legal), then making a new lex
		// state is required.
		int error_line = input_stream.getEndLine();
		int error_column = input_stream.getEndColumn();
		error_column -= image.length();
		throw new VrmlTokenMgrError(false, 0, error_line, error_column, 
		    null, image.charAt(0), VrmlTokenMgrError.LEXICAL_ERROR);
	    }
	    String idImage =  image.substring(0, dotIndex);
	    String fieldImage = image.substring(dotIndex+1);
	    Token retval = Token.newToken(ROUTE_USE_ID);
	    retval.kind = ROUTE_USE_ID;
	    retval.image = idImage;
	    // rows and cols for these tokens are probably not needed, but
	    // we'll give it a try.  
	    // beginLine == endLine (since the token can't contain a newline)
	    retval.beginLine = input_stream.getBeginLine();
	    retval.beginColumn = input_stream.getBeginColumn();
	    retval.endLine = retval.beginLine;
	    retval.endColumn = input_stream.getEndColumn();
	    retval.endColumn -= fieldImage.length() + 1;
	    Token dot = Token.newToken(DOT);
	    dot.kind = DOT;
	    dot.image = new String(".");
	    dot.beginLine = retval.beginLine;
	    dot.beginColumn = retval.beginColumn + idImage.length();
	    dot.endLine = dot.beginLine;
	    dot.endColumn = dot.beginColumn;
	    retval.next = dot;
	    Token fname = Token.newToken(FNAME);
	    fname.kind = FNAME;
	    fname.image = fieldImage;
	    fname.beginLine = retval.beginLine;
	    fname.beginColumn = retval.beginColumn + idImage.length() + 1;
	    fname.endLine = retval.beginLine;
	    fname.endColumn = retval.endColumn - fieldImage.length();
	    dot.next = fname;
	    lexMode = DEFAULT;
	    return retval;
	  case FIELD_NAME:
	    jjmatchedKind = FNAME;
	    lexMode = DEFAULT;
	    return jjFillToken(image);
	  default:
	    throw new Error("unknown lex state");
	}
    }

    public final Token getNextToken() {
	int kind;
	Token matchedToken;
	int curPos = 0;

	EOFLoop :
	for (;;) {   
	    try   {     
		curChar = input_stream.BeginToken();
	    } catch(java.io.IOException e) {        
		jjmatchedKind = 0;
		matchedToken = jjFillToken();
		return matchedToken;
	    }

	    try { 
		// advance the stream until past a "SKIP" character
		// SKIP chars = '\t', '\n', '\r', ' ', ',' 
		while (curChar <= 44 && 
				(0x100100003600L & (1L << curChar)) != 0L) {
		    curChar = input_stream.BeginToken();
		}
	    } catch (java.io.IOException e1) { 
		continue EOFLoop; 
	    }

	    // if curChar is now "#" then skip to end of line and continue
	    // at top of loop
	    if (curChar == 35) {
		try { 
		    while ((curChar != '\r') && (curChar != '\n')) {
			curChar = input_stream.BeginToken();
		    }
		} catch (java.io.IOException e1) { 
		    continue EOFLoop; 
		}
		continue EOFLoop;
	    } else if (curChar == '"') {
		jjmatchedKind = STRING_LITERAL;
		jjmatchedPos = 0;
		do {
		    curPos++;
		    jjmatchedPos++;
		    try { 
			curChar = input_stream.readChar(); 
		    } catch(java.io.IOException e) { 
			// Hmm, EOF in string literal, what kind of error 
			// is right here?  For now:
			continue EOFLoop;
		    }
		} while (curChar != '"');
		// done parsing the string
	    } else {
		// Have a non-SKIP, non-quote character, see if we match a token
		jjmatchedKind = NO_MATCH;
		jjmatchedPos = 0;
		curPos = jjMatchToken();
	    }

	    if (jjmatchedKind != NO_MATCH) {
		// Got a match 
		if ((jjmatchedPos + 1) < curPos) {
		    // curPos is past end of token, backup 
		    input_stream.backup(curPos - jjmatchedPos - 1);
		}

		// resolve a multi-char strings into tokens
		if (jjmatchedKind == MULTI_CHAR) {
		    String image = input_stream.GetImage();
		    char firstChar = image.charAt(0);
		    if (firstChar < 64) {
			long l = 1L << firstChar;
			// matches: '+', '-', '.', '0'-'9''
			if ((0x03ff680000000000L & l) != 0L) {
			    jjmatchedKind = NUMBER_LITERAL;
			    matchedToken = jjFillToken(image);
			} else if (firstChar == '\"') {
			    jjmatchedKind = STRING_LITERAL;
			    matchedToken = jjFillToken(image);
			} else {
			    matchedToken = processGeneralString(image);
			}
		    } else {
			matchedToken = processGeneralString(image);
		    }
		} else {
		    matchedToken = jjFillToken();
		}

		return matchedToken;
	    }

	    // Got a error, report it 
	    int error_line = input_stream.getEndLine();
	    int error_column = input_stream.getEndColumn();
	    String error_after = null;
	    boolean EOFSeen = false;
	    try { 
		input_stream.readChar(); 
		input_stream.backup(1); 
	    } catch (java.io.IOException e1) {
		EOFSeen = true;
		error_after = curPos <= 1 ? "" : input_stream.GetImage();
		if (curChar == '\n' || curChar == '\r') {
		    error_line++;
		    error_column = 0;
		} else {
		    error_column++;
		}
	    }
	    if (!EOFSeen) {
		input_stream.backup(1);
		error_after = curPos <= 1 ? "" : input_stream.GetImage();
	    } throw new VrmlTokenMgrError(EOFSeen, 0, error_line, error_column, 
		    error_after, curChar, VrmlTokenMgrError.LEXICAL_ERROR);
	}
    }

}
