/******************************************************************************
 *
 *                      VRML Browser basic classes
 *                   For External Authoring Interface
 *
 *                   (C) The VRML Consortium
 *
 *  Written by Justin Couch: justin@vlc.com.au
 *
 *****************************************************************************/

package vrml.external.field;

import vrml.external.Node;

/**
 * VRML eventIn class for MFNode.
 * <P>
 * Get the values of a node array. The java <CODE>null</CODE> reference is
 * treated to be equivalent to the VRML <CODE>NULL</CODE> field values.
 * <P>
 * It is not illegal to construct an array where some members of the array are
 * null pointers. Due to no specification on the intended result in the VRML
 * specification, the response given by the browser is implementation
 * dependent. Calls will not generate an exception, but the value of actual
 * event received from the scenegraph may vary until the issue is resolved.
 *
 * @deprecated
 * @version 1.0
 */
public class EventOutMFNode extends EventOutMField
{
  /**
   * Get the value of the array of nodes.
   * <P>
   *
   * @return The array of node references
   */
  public Node[] getValue()
  {
    return null;
  }

  /**
   * Get a particular node value in the given eventOut array.
   * <P>
   * If the index is out of the bounds of the current array of data values an
   * ArrayIndexOutOfBoundsException will be generated. If the Node value is
   * NULL because the browser implementation keeps null references then this
   * method will return a null pointer without throwing any exception.
   * <P>
   * @param index The position to read the values from
   * @return The node reference
   *
   * @exception ArrayIndexOutOfBoundsException The index was outside the current data
   *    array bounds.
   */
  public Node get1Value(int index)
  {
    return null;
  }
}











