/*
 *      @(#)Script.java 1.18 98/11/05 20:41:14
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 *
 * @Author: Rick Goldberg
 * @Author: Doug Gehringer
 */

package vrml.node;
import vrml.Field;
import vrml.Event;
import vrml.InvalidFieldException;
import vrml.InvalidEventInException;
import vrml.InvalidEventOutException;
import vrml.InvalidExposedFieldException;

/** 
 * This is the base class for VRML scripts.  
 */
public class Script extends vrml.BaseNode {

    vrml.Browser browser;
    com.sun.j3d.loaders.vrml97.impl.Script impl;

    public Script() {
	super(null);
	browser = null;
	impl = null;
    }

    /** This is the internal constructor, called only when the Script is
     * being initialized by the Loader.
     */
    public Script(com.sun.j3d.loaders.vrml97.impl.Script init) {
	super(init);
	impl = init;
	// 
	browser = new vrml.Browser(impl.getBrowser());
    }

    public Object clone() {
       return new Script((com.sun.j3d.loaders.vrml97.impl.Script)impl.clone());
    }


    public String getType() { return "Script" ; }
    public vrml.Browser getBrowser()  { return browser; }

    /** This is an internal method, used to link the Script to its
     * implementaton
     */
    public synchronized void registerOwner(com.sun.j3d.loaders.vrml97.impl.Script s) {
	impl = s;
	browser = new vrml.Browser(impl.getBrowser());
    }

    /**
     * Returns the Field specified in the VRML which invokes the script
     */
    protected synchronized final Field getField(String fieldName) 
     throws InvalidFieldException {
	return impl.getField(fieldName).wrap(); 
    }

    /**
     * Returns the EventOut specified in the VRML which invokes the script
     */
    protected synchronized final Field getEventOut(String eventOutName) 
     throws InvalidEventOutException {
	return impl.getEventOut(eventOutName).wrap();
    }

    /**
     * Returns the EventIn specified in the VRML which invokes the script
     */
    protected synchronized final Field getEventIn(String eventInName) 
     throws InvalidEventInException {
	return impl.getEventIn(eventInName).wrap();
    }

    /** Called before any event is generated */
    public void initialize() {;} // doesn't have to be abOutstract

    /** Called when there are Events to be processed */
    public void processEvents(int count, Event[] events ) {;}

    /** Called when there is an Event to be processed */
    public void processEvent(Event event) {;}

    /** Called after every invocation of processEvents() */
    public void eventsProcessed() {;}

    /** Called when the Script node is deleted */
    public void shutdown() {} 
}

